unit main_cli;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Buttons, IdBaseComponent, IdComponent, IdTCPConnection,
  IdTCPClient, Hash, HCMngr, ExtCtrls;

type
  TfmClient = class(TForm)
    GroupBox1: TGroupBox;
    etNickname: TEdit;
    Label1: TLabel;
    Label2: TLabel;
    etServerIP: TEdit;
    sbConnetti: TSpeedButton;
    TCPClient: TIdTCPClient;
    mmChat: TMemo;
    etSendText: TEdit;
    etPassword: TEdit;
    Label3: TLabel;
    cmCipher: TCipherManager;
    Label4: TLabel;
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure etSendTextKeyUp(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure sbConnettiClick(Sender: TObject);
    procedure etPasswordChange(Sender: TObject);
    procedure FormCreate(Sender: TObject);
  private
    { Private declarations }
    FPass: string;
    FCrypt: boolean;
  public
    { Public declarations }
    Nick: string;
    procedure Connect;
    procedure Disconnect;
  end;

  TRecvThread = class(TThread)
  private
    Msg: string;
    procedure NewMail;
  protected
    procedure Execute; override;
  end;

var
  fmClient: TfmClient;
  RecvThread: TRecvThread;

implementation

uses StrUtils;

{$R *.DFM}

procedure TfmClient.Connect;
begin
  try
    TCPClient.Host:=etServerIP.Text;
    Nick:=etNickname.Text;
    TCPClient.Connect;
    RecvThread:=TRecvThread.Create(false);
    RecvThread.FreeOnTerminate:=true;
    TCPClient.WriteLn(format('%s  entrato in chat',[Nick]));
  except
    on E:Exception do MessageBoxA(handle,pchar(format('Impossibile connettersi: %s',[E.Message])),'Errore',MB_ICONSTOP);
  end;
end;

procedure TfmClient.Disconnect;
begin
  if TCPClient.Connected then
  begin
    if Assigned(RecvThread) then
      TCPClient.WriteLn(format('%s  uscito dalla chat',[Nick]));
      begin
      RecvThread.Terminate;
      repeat
        Application.ProcessMessages;
      until RecvThread.Terminated;
    end;
    TCPClient.Disconnect;
  end;
end;

procedure TfmClient.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  Disconnect;
  Action:=caFree;
end;

procedure TfmClient.etSendTextKeyUp(Sender: TObject; var Key: Word;
  Shift: TShiftState);
var SendMessage:string;
begin
  if TCPClient.Connected then
    if key=13 then
    begin
      SendMessage:=etSendText.Text;
      if FCrypt then
        SendMessage:=cmCipher.EncodeString(SendMessage);
      TCPClient.WriteLn(format('%s> %s',[Nick,SendMessage]));
      etSendText.Clear;
    end;
end;

procedure TfmClient.sbConnettiClick(Sender: TObject);
begin
  if not TCPClient.Connected then
    Connect
  else
    Disconnect;
  sbConnetti.Down:=TCPClient.Connected;
end;

procedure TfmClient.etPasswordChange(Sender: TObject);
begin
  if Length(etPassword.Text)>0 then
  begin
    FCrypt:=true;
    FPass:=THash_MD5.CalcString(etPassword.Text);
    FPass:=LeftStr(FPass,22);
    cmCipher.InitKey(FPass,nil);
  end
  else
    FCrypt:=false;
end;

procedure TfmClient.FormCreate(Sender: TObject);
begin
  FCrypt:=false;
end;

{ TRecvThread }

procedure TRecvThread.Execute;
begin
  while not Terminated do
  begin
    try
      Msg:=fmClient.TCPClient.ReadLn;
      Synchronize(NewMail);
    except
      fmClient.sbConnetti.Down:=false;
      fmClient.TCPClient.Disconnect;
      Terminate;
    end;
  end;
end;

procedure TRecvThread.NewMail;
var mex, prefix:string;
    start:integer;
begin
  mex:=msg;
  start:=Pos('> ',mex)+1;
  if Start>1 then
  begin
    prefix:=LeftStr(mex,start);
    mex:=RightStr(mex,Length(mex)-(start));
    try
      if fmClient.FCrypt then
        mex:=fmClient.cmCipher.DecodeString(mex);
    except
    end;
    mex:=Format('%s%s',[prefix,mex]);
  end;
  fmClient.mmChat.Lines.Add(mex);
end;

end.
